/**
*<pre>
* ----------------------------------------------------------------------------
* Copyright Notice
* This file contains proprietary information of COMPANY.
* Copying or reproduction is allowed provided COMPANY is acknowledged in any
* copyright notice.
* The software in this file is not guaranteed to perform as specified but 
* COMPANY has done its best to ensure that it does behave as specified.
* Copyright (c) 2005
* ----------------------------------------------------------------------------
*</pre>
*
* <b>COMPANY code</p>
*
* www.COMPANY.com (support@COMPANY.com)
*
* @author Ricci Stefano
* @version 1.0
* @date 2005
*/  

MONTHS=["Gennaio","Febbraio","Marzo","Aprile","Maggio","Giugno","Luglio","Agosto","Settembre",
	"Ottobre","Novembre","Dicembre"];

NULL_VALUE_MONTHS_DEFAULT='---';
NULL_VALUE_MONTHS_TEXT='mese';
NULL_VALUE_DAYS_DEFAULT='--';
NULL_VALUE_DAYS_TEXT='giorno';

/*
	Oggetto dateField
	oggetto per gestire due campi select con mese(+anno) e giorno
	gestisce al massimo 12 mesi
*/


/**
* constructor
* @param arg description...
* @return description...
*/


function dateField(dayField, monthField, startingDate, endingDate, selectedDate, textNullValue)
{
	this.dayField=dayField;
	this.monthField=monthField;
	
	//functions
	this.populate = populate;
	this.update = update;
	this.setRange = setRange;
	this.setStartingDate = setStartingDate;
	this.setSelectedDate = setSelectedDate;
	this.setEndingDate = setEndingDate;
	this.setStartRange = setStartRange;
	this.setEndRange = setEndRange;
	this.getSelectedDate = getSelectedDate;
	this.formatDate=formatDate;

	this.setStartingDate(startingDate);
	this.setEndingDate(endingDate);
	
	this.startRange=this.startingDate;
	this.endRange=this.endingDate;

	this.setSelectedDate(selectedDate);
	this.value="";
	if(textNullValue!=null && textNullValue){
		this.monthsNullValue=NULL_VALUE_MONTHS_TEXT;
		this.daysNullValue=NULL_VALUE_DAYS_TEXT;
	}	else {
		this.monthsNullValue=NULL_VALUE_MONTHS_DEFAULT;
		this.daysNullValue=NULL_VALUE_DAYS_DEFAULT;
	}
}

function populate()
{
	this.monthField.disabled=false;
	this.dayField.disabled=false;
	
	var start;
	var end;
	var selected=this.selectedDate;
	
	if(this.startRange==null)
		start=this.startingDate;
	else start=this.startRange;
	
	if(this.endRange==null)
		end=this.endingDate;
	else end=this.endRange;
	
	var year=Number(start[0]);
	var totalMonths=0;
	
	//calculate total months showed
	for(var i=0;i<14;i++)
		if(this.monthField.options[i]==null){
			totalMonths=i+1;
			break;
		}
	//set month options to null
	for(var i=totalMonths-1;i>=0;i--)
		this.monthField.options[i]=null;

	this.monthField.options[0] = new Option(this.monthsNullValue,0);

	if(selected==null)
		this.monthField.options[0].selected=true;
	var i=0;
	for(i=1,month=Number(start[1]); 
		(end!=null && compareDates([year,month,1],[end[0],end[1],1]) >= 0) && i<13; i++,month++) {
		this.monthField.options[i] = new Option(MONTHS[month-1]+" "+year,year+(month<10?"0"+String(month):String(month)));
		if(selected!=null && year==selected[0] && month==selected[1])
			this.monthField.options[i].selected=true;
		if(month==12) {
			month=0;
			year++;
		}
	}
	if(i==1)
		this.monthField.disabled=true;
		
	//init day options

	this.dayField.options[0] = new Option(this.daysNullValue,0);
	if(selected==null || selected[2]==0)
		this.dayField.options[0].selected=true;
	
	for(var i=32;i>0;i--)
		this.dayField.options[i]=null;

	if(selected != null) {
		//set total days in list
		var totalDays=0;
		if(selected[1] == 0)
			totalDays=0;
		else if( selected[1] == 2 ) {
			if(selected[0] % 400 ==0 || (selected[0] % 4 == 0 && selected[0] % 100 !=0))
				totalDays=29;
			else 
				totalDays=28;
		} else if( selected[1]==9 || selected[1]==4 || 
				selected[1]==6 || selected[1]==11)
			totalDays=30;
		else
			totalDays=31;

		//set possible days in list
		var dayIndex=0;
		var startingDay=1;
		dayIndex++;

		// inferior limit to days
		if(start[0]==selected[0] && start[1]==selected[1] && 	start[2]!=0)
			startingDay=start[2];
		if(end[0]==selected[0] && end[1]==selected[1] && 	end[2]!=0)
			totalDays=end[2];

		if(selected[2]>totalDays)
			selected[2]=totalDays;

		if(totalDays==0)
			this.dayField.disabled=true;
		
		for (var i=startingDay; i <=totalDays; i++,dayIndex++) {
			var x= String(i);
			this.dayField.options[dayIndex] = new Option(x,x);
			if(x==selected[2])
				this.dayField.options[dayIndex].selected=true;
		}
		this.selectedDate=selected;
	} else this.dayField.disabled=true;
	this.value=this.formatDate(selected);
}

function update()
{
	var date=this.getSelectedDate();
	this.selectedDate=date;

	this.populate();
}

function setStartingDate(startingDate)
{
	if(startingDate == null || startingDate[0]==0) {
		var d=new Date();
		var day = d.getDate();
		var month = d.getMonth()+1;
		var year = d.getYear();
		this.startingDate = [year,month,day];
	} else this.startingDate=startingDate;
}

function setEndingDate(endingDate)
{
	if(endingDate == null || endingDate[0]==0){
		var d=new Date();
		var day = 31;
		var month = d.getMonth();
		var year = d.getYear();
		var date1;
		do {
			date1=new Date(year+1,month-1,day--);
		}
		while(day>=0 && date1.getMonth()!=(month-1));
		this.endingDate=[date1.getYear(),date1.getMonth()+1,date1.getDate()];
	}
	else this.endingDate=endingDate;
	if(compareDates(this.endingDate,this.startingDate)==1)
		this.endingDate=this.startingDate;
}

function setRange(startRange, endRange)
{
	this.setStartRange(startRange);
	this.setEndRange(endRange);

	this.setSelectedDate(this.selectedDate);
}

function setStartRange(startRange)
{
	if(startRange==null || startRange[0]==0)
		this.startRange=this.startingDate;
	else if(compareDates(startRange,this.startingDate)==1)
		this.startRange=this.startingDate;
	else if(compareDates(startRange,this.endingDate)==-1)
		this.startRange=this.endingDate;
	else this.startRange=startRange;
}

function setEndRange(endRange)
{
	if(endRange==null || endRange[0]==0)
		this.endRange=this.endingDate;
	else if(compareDates(endRange,this.startingDate)==1)
		this.endRange=this.startingDate;
	else if(compareDates(endRange,this.endingDate)==-1)
		this.endRange=this.endingDate;
	else this.endRange=endRange;
}

function setSelectedDate(selectedDate) 
{
	if(selectedDate!=null && this.selectedDate!=null && this.selectedDate[0]!=0){
		if(compareDates(this.startRange, selectedDate) == -1 && this.selectedDate[2]!=0)
			this.selectedDate=this.startRange;
		else if(compareDates(this.endRange, selectedDate) == 1 && this.selectedDate[2]!=0)
			this.selectedDate=this.endRange;
		else this.selectedDate = selectedDate;
	}	else this.selectedDate = selectedDate;
}


function getSelectedDate()
{
	var yearAndMonth=this.monthField.options[this.monthField.selectedIndex].value;
	var year=0;
	var month=0;
	var day=0;
	if(yearAndMonth!=null && yearAndMonth!=0){
		year=yearAndMonth.substring(0,4);
		month=yearAndMonth.substring(4);
	}
	day=this.dayField.options[this.dayField.selectedIndex].value;
	var date=[year,month,day];
	return date;
}

function formatDate(date){
	if(date!=null)
		return date[0]+"/"+date[1]+"/"+date[2];
	else return "";
}
//object relatedField

function relatedField(startingField, endingField, min)
{
	this.startingField=startingField;
	this.endingField=endingField;
	this.MIN_INTERVAL=min;
	
	//functions
	this.startChanged=startChanged;
	this.endChanged=endChanged;
}

function startChanged()
{
	this.startingField.update();
	var date=this.startingField.getSelectedDate();
	if(date[0]==0)
		this.endingField.setRange(null,null);
	else {
		if(date[2]==0) {
			if(date[0]==this.startingField.startRange[0] &&
				date[1]==this.startingField.startRange[1] && 
				this.startingField.startRange[2]!=0)
				date[2]=this.startingField.startRange[2];				
			else date[2]=1;
		}
		var date1=new Date(date[0],date[1]-1,date[2]);
		var date2=new Date(date1.getYear(),date1.getMonth(),date1.getDate()+this.MIN_INTERVAL);
		var startRange=[date2.getYear(),date2.getMonth()+1,date2.getDate()];
		this.endingField.setRange(startRange,null);
	}
	this.endingField.populate();
}

function endChanged()
{
	this.endingField.update();
	var date=this.endingField.getSelectedDate();
	if(date[0]==0)
		this.startingField.setRange(null,null);
	else {
		if(date[2]==0){
		 	if(date[0]==this.endingField.endRange[0] &&
				date[1]==this.endingField.endRange[1] && 
				this.endingField.endRange[2]!=0)
				date[2]=this.endingField.endRange[2];
			else date[2]=31;
		}
		var date1=new Date(date[0],date[1]-1,date[2]);
		var date2=new Date(date1.getYear(),date1.getMonth(),date1.getDate()-this.MIN_INTERVAL);
		var endRange=[date2.getYear(),date2.getMonth()+1,date2.getDate()];
		this.startingField.setRange(null,endRange);
	}
	this.startingField.populate();
}

//returns 0 if equal, -1 if date1 greater than date 2, 1 in the other case
function compareDates(date1, date2)
{
	if(date1 == null) {
		if(date2==null)
			return 0;
		else return 1;
	} else if(date2 == null)
		return -1;
	else {
		if(date1[0]==0){
			if(date2[0]==0) return 0;
			else return -1;
		} else if(date2[0]==0) return 1;
		else if(date1[0]>date2[0]) {
			if(date2[0]==0)
				return 1;
			else return -1;
		}	else if(date1[0]==date2[0]) {
			if(date1[1]==0){
				if(date2[1]==0) return 0;
				else return -1;
			} else if(date2[1]==0) return 1;
			else if(date1[1] > date2[1]){
				if(date2[1]==0)
					return 1;
				else return -1;
			}	else if(date1[1] == date2[1]) {
				if(date1[2]==0){
					if(date2[2]==0) return 0;
					else return -1;
				} else if(date2[2]==0) return 1;
				else if(date1[2] > date2[2]){
					if(date2[2]==0)
						return 1;
					else return -1;
				} else if(date1[2] == date2[2])
					return 0;
				else return 1;
			} else return 1;
		} else return 1;
	}
}